==> sdk-4.0.0-sources.jar: com/kontakt/sdk/android/ble/service/MonitorCallback.java <==
package com.kontakt.sdk.android.ble.service;

import android.bluetooth.BluetoothDevice;
import android.os.Handler;

import com.kontakt.sdk.android.ble.cache.FutureShufflesCache;
import com.kontakt.sdk.android.ble.configuration.ActivityCheckConfiguration;
import com.kontakt.sdk.android.ble.configuration.ScanContext;
import com.kontakt.sdk.android.ble.configuration.ScanPeriod;
import com.kontakt.sdk.android.ble.discovery.BluetoothDeviceDiscoverer;
import com.kontakt.sdk.android.ble.discovery.BluetoothDeviceEvent;
import com.kontakt.sdk.android.ble.discovery.DiscoveryContract;
import com.kontakt.sdk.android.ble.discovery.EventType;
import com.kontakt.sdk.android.ble.exception.ScanError;
import com.kontakt.sdk.android.ble.manager.listeners.InternalProximityListener;
import com.kontakt.sdk.android.ble.monitoring.IEventCollector;
import com.kontakt.sdk.android.common.KontaktSDK;
import com.kontakt.sdk.android.common.log.Logger;
import com.kontakt.sdk.android.common.profile.RemoteBluetoothDevice;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

@SuppressWarnings({"MissingPermission", "WeakerAccess", "DefaultLocale"})
abstract class MonitorCallback implements BleScanCallback, DiscoveryContract {

  static MonitorCallback create(ScanContext scanContext, FutureShufflesCache cache, IEventCollector eventCollector) {
    return new MonitorCallback(scanContext, cache, eventCollector) {
    };
  }

  final BluetoothDeviceDiscoverer bluetoothDeviceDiscoverer;
  private final Handler handler = new Handler();
  private final Map<Integer, InternalProximityListener> monitoringListenerMap = new ConcurrentHashMap<>();
  private final ActivityCheckConfiguration activityCheckConfiguration;
  private final ScanPeriod scanPeriod;
  private final IEventCollector eventCollector;

  private MonitorCallback(ScanContext scanContext, FutureShufflesCache cache, IEventCollector eventCollector) {
    this.eventCollector = eventCollector;
    this.bluetoothDeviceDiscoverer = new DefaultBluetoothDeviceDiscoverer(scanContext, this, cache);
    this.activityCheckConfiguration = scanContext.getActivityCheckConfiguration();
    this.scanPeriod = scanContext.getScanPeriod();
  }

  @Override
  public void onLeScan(final BluetoothDevice device, final int rssi, final byte[] scanRecord) {
    if (!isAnyListenerRegistered() && KontaktSDK.isInitialized()) {
      return;
    }

    if (device == null) {
      Logger.d("Remote device discovered but is null");
      return;
    }

    Logger.d(String.format("Device discovered: %s Payload: %s RSSI: %d", device.toString(), Arrays.toString(scanRecord), rssi));
    performDiscovery(device, rssi, scanRecord);
  }

  private void performDiscovery(final BluetoothDevice device, final int rssi, final byte[] scanRecord) {
    handler.post(new Runnable() {
      @Override
      public void run() {
        bluetoothDeviceDiscoverer.performDiscovery(device, rssi, scanRecord);
      }
    });
  }

  @Override
  public void addListener(InternalProximityListener proximityListener) {
    final int hashCode = System.identityHashCode(proximityListener);
    final InternalProximityListener insertion = monitoringListenerMap.put(hashCode, proximityListener);

    if (insertion == null) {
      Logger.d("MonitoringListener registered: " + hashCode);
    }
  }

  @Override
  public void removeListener(InternalProximityListener proximityListener) {
    final int hashCode = System.identityHashCode(proximityListener);
    final InternalProximityListener deletion = monitoringListenerMap.remove(hashCode);

    if (deletion != null) {
      Logger.d("MonitoringListener unregistered: " + hashCode);
    }
  }

  @Override
  public Collection<InternalProximityListener> getMonitoringListeners() {
    return Collections.unmodifiableCollection(new ArrayList<>(monitoringListenerMap.values()));
  }

  @Override
  public void onMonitorCycleStart() {
    if (scanPeriod != ScanPeriod.RANGING) {
      for (final InternalProximityListener proximityListener : monitoringListenerMap.values()) {
        proximityListener.onMonitoringCycleStart();
      }
    }
    handler.post(new Runnable() {
      @Override
      public void run() {
        startPeriodicInactivityCheck();
      }
    });
  }

  @Override
  public void onMonitorCycleStop() {
    if (scanPeriod != ScanPeriod.RANGING) {
      for (final InternalProximityListener proximityListener : monitoringListenerMap.values()) {
        proximityListener.onMonitoringCycleStop();
      }
    }
    handler.post(new Runnable() {
      @Override
      public void run() {
        stopPeriodicInactivityCheck();
      }
    });
  }

  @Override
  public void onEvent(BluetoothDeviceEvent event) {
    for (final InternalProximityListener proximityListener : monitoringListenerMap.values()) {
      proximityListener.onEvent(event);
    }
    collectEvent(event);
  }

  @Override
  public void onScanError(ScanError exception) {
    for (final InternalProximityListener proximityListener : monitoringListenerMap.values()) {
      proximityListener.onScanError(exception);
    }
  }

  private void collectEvent(BluetoothDeviceEvent event) {
    EventType type = event.getEventType();
    if (type == EventType.DEVICE_DISCOVERED || type == EventType.DEVICES_UPDATE) {
      List<? extends RemoteBluetoothDevice> devices = event.getDeviceList();
      for (RemoteBluetoothDevice device : devices) {
        eventCollector.collect(device);
      }
    }
  }

  void startPeriodicInactivityCheck() {
    if (activityCheckConfiguration.equals(ActivityCheckConfiguration.DISABLED)) {
      return;
    }
    final long checkPeriod = activityCheckConfiguration.getCheckPeriod();
    handler.postDelayed(periodicActivityCheckRunnable, checkPeriod);
  }

  void stopPeriodicInactivityCheck() {
    if (activityCheckConfiguration == ActivityCheckConfiguration.DISABLED) {
      return;
    }
    handler.removeCallbacksAndMessages(null);
  }

  void evictInactiveDevices(long currentTimeMillis) {
    bluetoothDeviceDiscoverer.evictInactiveDevices(currentTimeMillis);
    startPeriodicInactivityCheck();
  }

  private boolean isAnyListenerRegistered() {
    return !monitoringListenerMap.isEmpty();
  }

  @Override
  public void close() {
    eventCollector.stop();
    eventCollector.clear();
    bluetoothDeviceDiscoverer.disable();
    monitoringListenerMap.clear();
    handler.removeCallbacksAndMessages(null);
  }

  private final Runnable periodicActivityCheckRunnable = new Runnable() {
    @Override
    public void run() {
      evictInactiveDevices(System.currentTimeMillis());
    }
  };

}

==> sdk-5.0.1-sources.jar: com/kontakt/sdk/android/ble/service/MonitorCallback.java <==
package com.kontakt.sdk.android.ble.service;

import android.bluetooth.BluetoothDevice;
import android.os.Handler;
import android.util.Log;

import com.kontakt.sdk.android.ble.cache.FutureShufflesCache;
import com.kontakt.sdk.android.ble.configuration.ActivityCheckConfiguration;
import com.kontakt.sdk.android.ble.configuration.ScanContext;
import com.kontakt.sdk.android.ble.configuration.ScanPeriod;
import com.kontakt.sdk.android.ble.discovery.BluetoothDeviceDiscoverer;
import com.kontakt.sdk.android.ble.discovery.BluetoothDeviceEvent;
import com.kontakt.sdk.android.ble.discovery.DiscoveryContract;
import com.kontakt.sdk.android.ble.discovery.EventType;
import com.kontakt.sdk.android.ble.exception.ScanError;
import com.kontakt.sdk.android.ble.manager.listeners.InternalProximityListener;
import com.kontakt.sdk.android.ble.monitoring.IEventCollector;
import com.kontakt.sdk.android.common.KontaktSDK;
import com.kontakt.sdk.android.common.log.Logger;
import com.kontakt.sdk.android.common.profile.RemoteBluetoothDevice;
import com.kontakt.sdk.android.common.util.ArrayUtils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

@SuppressWarnings({"MissingPermission", "WeakerAccess", "DefaultLocale"})
abstract class MonitorCallback implements BleScanCallback, DiscoveryContract {

  private static final String TAG = MonitorCallback.class.getSimpleName();

  static MonitorCallback create(ScanContext scanContext, FutureShufflesCache cache, IEventCollector eventCollector) {
    return new MonitorCallback(scanContext, cache, eventCollector) {
    };
  }

  final BluetoothDeviceDiscoverer bluetoothDeviceDiscoverer;
  private final Handler handler = new Handler();
  private final Map<Integer, InternalProximityListener> monitoringListenerMap = new ConcurrentHashMap<>();
  private final ActivityCheckConfiguration activityCheckConfiguration;
  private final ScanPeriod scanPeriod;
  private final IEventCollector eventCollector;

  private MonitorCallback(ScanContext scanContext, FutureShufflesCache cache, IEventCollector eventCollector) {
    this.eventCollector = eventCollector;
    this.bluetoothDeviceDiscoverer = new DefaultBluetoothDeviceDiscoverer(scanContext, this, cache);
    this.activityCheckConfiguration = scanContext.getActivityCheckConfiguration();
    this.scanPeriod = scanContext.getScanPeriod();
  }

  @Override
  public void onLeScan(final BluetoothDevice device, final int rssi, final byte[] scanRecord) {
    if (!isAnyListenerRegistered() && KontaktSDK.isInitialized()) {
      Log.d(TAG, "Got scan, but SDK is not initialized or no listeners are registered");
      return;
    }

    if (device == null) {
      Log.d(TAG, "Got scan, but SDK is not initialized or no listeners are registered");
      Logger.d("Remote device discovered but is null");
      return;
    }

    Logger.d(String.format("Device discovered: %s Payload: %s RSSI: %d", device.toString(), Arrays.toString(scanRecord), rssi));
    ArrayUtils.printByteArray(scanRecord, String.format("Device bytes discovered, mac: %s", device.getAddress()));
    performDiscovery(device, rssi, scanRecord);
  }

  private void performDiscovery(final BluetoothDevice device, final int rssi, final byte[] scanRecord) {
    handler.post(new Runnable() {
      @Override
      public void run() {
        bluetoothDeviceDiscoverer.performDiscovery(device, rssi, scanRecord);
      }
    });
  }

  @Override
  public void addListener(InternalProximityListener proximityListener) {
    final int hashCode = System.identityHashCode(proximityListener);
    final InternalProximityListener insertion = monitoringListenerMap.put(hashCode, proximityListener);

    if (insertion == null) {
      Logger.d("MonitoringListener registered: " + hashCode);
    }
  }

  @Override
  public void removeListener(InternalProximityListener proximityListener) {
    final int hashCode = System.identityHashCode(proximityListener);
    final InternalProximityListener deletion = monitoringListenerMap.remove(hashCode);

    if (deletion != null) {
      Logger.d("MonitoringListener unregistered: " + hashCode);
    }
  }

  @Override
  public Collection<InternalProximityListener> getMonitoringListeners() {
    return Collections.unmodifiableCollection(new ArrayList<>(monitoringListenerMap.values()));
  }

  @Override
  public void onMonitorCycleStart() {
    if (scanPeriod != ScanPeriod.RANGING) {
      for (final InternalProximityListener proximityListener : monitoringListenerMap.values()) {
        proximityListener.onMonitoringCycleStart();
      }
    }
    handler.post(new Runnable() {
      @Override
      public void run() {
        startPeriodicInactivityCheck();
      }
    });
  }

  @Override
  public void onMonitorCycleStop() {
    if (scanPeriod != ScanPeriod.RANGING) {
      for (final InternalProximityListener proximityListener : monitoringListenerMap.values()) {
        proximityListener.onMonitoringCycleStop();
      }
    }
    handler.post(new Runnable() {
      @Override
      public void run() {
        stopPeriodicInactivityCheck();
      }
    });
  }

  @Override
  public void onEvent(BluetoothDeviceEvent event) {
    for (final InternalProximityListener proximityListener : monitoringListenerMap.values()) {
      proximityListener.onEvent(event);
    }
    collectEvent(event);
  }

  @Override
  public void onScanError(ScanError exception) {
    for (final InternalProximityListener proximityListener : monitoringListenerMap.values()) {
      proximityListener.onScanError(exception);
    }
  }

  private void collectEvent(BluetoothDeviceEvent event) {
    EventType type = event.getEventType();
    if (type == EventType.DEVICE_DISCOVERED || type == EventType.DEVICES_UPDATE) {
      List<? extends RemoteBluetoothDevice> devices = event.getDeviceList();
      for (RemoteBluetoothDevice device : devices) {
        eventCollector.collect(device);
      }
    }
  }

  void startPeriodicInactivityCheck() {
    if (activityCheckConfiguration.equals(ActivityCheckConfiguration.DISABLED)) {
      return;
    }
    final long checkPeriod = activityCheckConfiguration.getCheckPeriod();
    handler.postDelayed(periodicActivityCheckRunnable, checkPeriod);
  }

  void stopPeriodicInactivityCheck() {
    if (activityCheckConfiguration == ActivityCheckConfiguration.DISABLED) {
      return;
    }
    handler.removeCallbacksAndMessages(null);
  }

  void evictInactiveDevices(long currentTimeMillis) {
    bluetoothDeviceDiscoverer.evictInactiveDevices(currentTimeMillis);
    startPeriodicInactivityCheck();
  }

  private boolean isAnyListenerRegistered() {
    return !monitoringListenerMap.isEmpty();
  }

  @Override
  public void close() {
    eventCollector.stop();
    eventCollector.clear();
    bluetoothDeviceDiscoverer.disable();
    monitoringListenerMap.clear();
    handler.removeCallbacksAndMessages(null);
  }

  private final Runnable periodicActivityCheckRunnable = new Runnable() {
    @Override
    public void run() {
      evictInactiveDevices(System.currentTimeMillis());
    }
  };

}
